# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .objects import (
    ObjectsResource,
    AsyncObjectsResource,
    ObjectsResourceWithRawResponse,
    AsyncObjectsResourceWithRawResponse,
    ObjectsResourceWithStreamingResponse,
    AsyncObjectsResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ...._base_client import AsyncPaginator, make_request_options
from ....types.durable_objects import namespace_list_params
from ....types.durable_objects.namespace import Namespace

__all__ = ["NamespacesResource", "AsyncNamespacesResource"]


class NamespacesResource(SyncAPIResource):
    @cached_property
    def objects(self) -> ObjectsResource:
        return ObjectsResource(self._client)

    @cached_property
    def with_raw_response(self) -> NamespacesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return NamespacesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> NamespacesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return NamespacesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[Namespace]:
        """
        Returns the Durable Object namespaces owned by an account.

        Args:
          account_id: Identifier.

          page: Current page.

          per_page: Items per-page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/workers/durable_objects/namespaces", account_id=account_id),
            page=SyncV4PagePaginationArray[Namespace],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    namespace_list_params.NamespaceListParams,
                ),
            ),
            model=Namespace,
        )


class AsyncNamespacesResource(AsyncAPIResource):
    @cached_property
    def objects(self) -> AsyncObjectsResource:
        return AsyncObjectsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncNamespacesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncNamespacesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncNamespacesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncNamespacesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Namespace, AsyncV4PagePaginationArray[Namespace]]:
        """
        Returns the Durable Object namespaces owned by an account.

        Args:
          account_id: Identifier.

          page: Current page.

          per_page: Items per-page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/workers/durable_objects/namespaces", account_id=account_id),
            page=AsyncV4PagePaginationArray[Namespace],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    namespace_list_params.NamespaceListParams,
                ),
            ),
            model=Namespace,
        )


class NamespacesResourceWithRawResponse:
    def __init__(self, namespaces: NamespacesResource) -> None:
        self._namespaces = namespaces

        self.list = to_raw_response_wrapper(
            namespaces.list,
        )

    @cached_property
    def objects(self) -> ObjectsResourceWithRawResponse:
        return ObjectsResourceWithRawResponse(self._namespaces.objects)


class AsyncNamespacesResourceWithRawResponse:
    def __init__(self, namespaces: AsyncNamespacesResource) -> None:
        self._namespaces = namespaces

        self.list = async_to_raw_response_wrapper(
            namespaces.list,
        )

    @cached_property
    def objects(self) -> AsyncObjectsResourceWithRawResponse:
        return AsyncObjectsResourceWithRawResponse(self._namespaces.objects)


class NamespacesResourceWithStreamingResponse:
    def __init__(self, namespaces: NamespacesResource) -> None:
        self._namespaces = namespaces

        self.list = to_streamed_response_wrapper(
            namespaces.list,
        )

    @cached_property
    def objects(self) -> ObjectsResourceWithStreamingResponse:
        return ObjectsResourceWithStreamingResponse(self._namespaces.objects)


class AsyncNamespacesResourceWithStreamingResponse:
    def __init__(self, namespaces: AsyncNamespacesResource) -> None:
        self._namespaces = namespaces

        self.list = async_to_streamed_response_wrapper(
            namespaces.list,
        )

    @cached_property
    def objects(self) -> AsyncObjectsResourceWithStreamingResponse:
        return AsyncObjectsResourceWithStreamingResponse(self._namespaces.objects)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .organization import Organization

__all__ = ["UserGetResponse"]


class UserGetResponse(BaseModel):
    id: Optional[str] = None
    """Identifier of the user."""

    betas: Optional[List[str]] = None
    """Lists the betas that the user is participating in."""

    country: Optional[str] = None
    """The country in which the user lives."""

    first_name: Optional[str] = None
    """User's first name"""

    has_business_zones: Optional[bool] = None
    """Indicates whether user has any business zones"""

    has_enterprise_zones: Optional[bool] = None
    """Indicates whether user has any enterprise zones"""

    has_pro_zones: Optional[bool] = None
    """Indicates whether user has any pro zones"""

    last_name: Optional[str] = None
    """User's last name"""

    organizations: Optional[List[Organization]] = None

    suspended: Optional[bool] = None
    """Indicates whether user has been suspended"""

    telephone: Optional[str] = None
    """User's telephone number"""

    two_factor_authentication_enabled: Optional[bool] = None
    """Indicates whether two-factor authentication is enabled for the user account.

    Does not apply to API authentication.
    """

    two_factor_authentication_locked: Optional[bool] = None
    """
    Indicates whether two-factor authentication is required by one of the accounts
    that the user is a member of.
    """

    zipcode: Optional[str] = None
    """The zipcode or postal code where the user lives."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["EndpointHealthcheckUpdateResponse"]


class EndpointHealthcheckUpdateResponse(BaseModel):
    check_type: Literal["icmp"]
    """type of check to perform"""

    endpoint: str
    """the IP address of the host to perform checks against"""

    id: Optional[str] = None
    """UUID."""

    name: Optional[str] = None
    """Optional name associated with this check"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncSinglePage, AsyncSinglePage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.workers import route_create_params, route_update_params
from ...types.workers.route_get_response import RouteGetResponse
from ...types.workers.route_list_response import RouteListResponse
from ...types.workers.route_create_response import RouteCreateResponse
from ...types.workers.route_delete_response import RouteDeleteResponse
from ...types.workers.route_update_response import RouteUpdateResponse

__all__ = ["RoutesResource", "AsyncRoutesResource"]


class RoutesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RoutesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RoutesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RoutesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RoutesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        zone_id: str,
        pattern: str,
        script: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RouteCreateResponse:
        """
        Creates a route that maps a URL pattern to a Worker.

        Args:
          zone_id: Identifier.

          pattern: Pattern to match incoming requests against.
              [Learn more](https://developers.cloudflare.com/workers/configuration/routing/routes/#matching-behavior).

          script: Name of the script to run if the route matches.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/workers/routes", zone_id=zone_id),
            body=maybe_transform(
                {
                    "pattern": pattern,
                    "script": script,
                },
                route_create_params.RouteCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[RouteCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[RouteCreateResponse], ResultWrapper[RouteCreateResponse]),
        )

    def update(
        self,
        route_id: str,
        *,
        zone_id: str,
        pattern: str,
        script: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RouteUpdateResponse:
        """
        Updates the URL pattern or Worker associated with a route.

        Args:
          zone_id: Identifier.

          route_id: Identifier.

          pattern: Pattern to match incoming requests against.
              [Learn more](https://developers.cloudflare.com/workers/configuration/routing/routes/#matching-behavior).

          script: Name of the script to run if the route matches.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not route_id:
            raise ValueError(f"Expected a non-empty value for `route_id` but received {route_id!r}")
        return self._put(
            path_template("/zones/{zone_id}/workers/routes/{route_id}", zone_id=zone_id, route_id=route_id),
            body=maybe_transform(
                {
                    "pattern": pattern,
                    "script": script,
                },
                route_update_params.RouteUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[RouteUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[RouteUpdateResponse], ResultWrapper[RouteUpdateResponse]),
        )

    def list(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[RouteListResponse]:
        """
        Returns routes for a zone.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/workers/routes", zone_id=zone_id),
            page=SyncSinglePage[RouteListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=RouteListResponse,
        )

    def delete(
        self,
        route_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RouteDeleteResponse:
        """
        Deletes a route.

        Args:
          zone_id: Identifier.

          route_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not route_id:
            raise ValueError(f"Expected a non-empty value for `route_id` but received {route_id!r}")
        return self._delete(
            path_template("/zones/{zone_id}/workers/routes/{route_id}", zone_id=zone_id, route_id=route_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[RouteDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[RouteDeleteResponse], ResultWrapper[RouteDeleteResponse]),
        )

    def get(
        self,
        route_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RouteGetResponse:
        """
        Returns information about a route, including URL pattern and Worker.

        Args:
          zone_id: Identifier.

          route_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not route_id:
            raise ValueError(f"Expected a non-empty value for `route_id` but received {route_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/workers/routes/{route_id}", zone_id=zone_id, route_id=route_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[RouteGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[RouteGetResponse], ResultWrapper[RouteGetResponse]),
        )


class AsyncRoutesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRoutesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRoutesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRoutesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRoutesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        zone_id: str,
        pattern: str,
        script: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RouteCreateResponse:
        """
        Creates a route that maps a URL pattern to a Worker.

        Args:
          zone_id: Identifier.

          pattern: Pattern to match incoming requests against.
              [Learn more](https://developers.cloudflare.com/workers/configuration/routing/routes/#matching-behavior).

          script: Name of the script to run if the route matches.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/workers/routes", zone_id=zone_id),
            body=await async_maybe_transform(
                {
                    "pattern": pattern,
                    "script": script,
                },
                route_create_params.RouteCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[RouteCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[RouteCreateResponse], ResultWrapper[RouteCreateResponse]),
        )

    async def update(
        self,
        route_id: str,
        *,
        zone_id: str,
        pattern: str,
        script: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RouteUpdateResponse:
        """
        Updates the URL pattern or Worker associated with a route.

        Args:
          zone_id: Identifier.

          route_id: Identifier.

          pattern: Pattern to match incoming requests against.
              [Learn more](https://developers.cloudflare.com/workers/configuration/routing/routes/#matching-behavior).

          script: Name of the script to run if the route matches.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not route_id:
            raise ValueError(f"Expected a non-empty value for `route_id` but received {route_id!r}")
        return await self._put(
            path_template("/zones/{zone_id}/workers/routes/{route_id}", zone_id=zone_id, route_id=route_id),
            body=await async_maybe_transform(
                {
                    "pattern": pattern,
                    "script": script,
                },
                route_update_params.RouteUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[RouteUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[RouteUpdateResponse], ResultWrapper[RouteUpdateResponse]),
        )

    def list(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[RouteListResponse, AsyncSinglePage[RouteListResponse]]:
        """
        Returns routes for a zone.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/workers/routes", zone_id=zone_id),
            page=AsyncSinglePage[RouteListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=RouteListResponse,
        )

    async def delete(
        self,
        route_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RouteDeleteResponse:
        """
        Deletes a route.

        Args:
          zone_id: Identifier.

          route_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not route_id:
            raise ValueError(f"Expected a non-empty value for `route_id` but received {route_id!r}")
        return await self._delete(
            path_template("/zones/{zone_id}/workers/routes/{route_id}", zone_id=zone_id, route_id=route_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[RouteDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[RouteDeleteResponse], ResultWrapper[RouteDeleteResponse]),
        )

    async def get(
        self,
        route_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RouteGetResponse:
        """
        Returns information about a route, including URL pattern and Worker.

        Args:
          zone_id: Identifier.

          route_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not route_id:
            raise ValueError(f"Expected a non-empty value for `route_id` but received {route_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/workers/routes/{route_id}", zone_id=zone_id, route_id=route_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[RouteGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[RouteGetResponse], ResultWrapper[RouteGetResponse]),
        )


class RoutesResourceWithRawResponse:
    def __init__(self, routes: RoutesResource) -> None:
        self._routes = routes

        self.create = to_raw_response_wrapper(
            routes.create,
        )
        self.update = to_raw_response_wrapper(
            routes.update,
        )
        self.list = to_raw_response_wrapper(
            routes.list,
        )
        self.delete = to_raw_response_wrapper(
            routes.delete,
        )
        self.get = to_raw_response_wrapper(
            routes.get,
        )


class AsyncRoutesResourceWithRawResponse:
    def __init__(self, routes: AsyncRoutesResource) -> None:
        self._routes = routes

        self.create = async_to_raw_response_wrapper(
            routes.create,
        )
        self.update = async_to_raw_response_wrapper(
            routes.update,
        )
        self.list = async_to_raw_response_wrapper(
            routes.list,
        )
        self.delete = async_to_raw_response_wrapper(
            routes.delete,
        )
        self.get = async_to_raw_response_wrapper(
            routes.get,
        )


class RoutesResourceWithStreamingResponse:
    def __init__(self, routes: RoutesResource) -> None:
        self._routes = routes

        self.create = to_streamed_response_wrapper(
            routes.create,
        )
        self.update = to_streamed_response_wrapper(
            routes.update,
        )
        self.list = to_streamed_response_wrapper(
            routes.list,
        )
        self.delete = to_streamed_response_wrapper(
            routes.delete,
        )
        self.get = to_streamed_response_wrapper(
            routes.get,
        )


class AsyncRoutesResourceWithStreamingResponse:
    def __init__(self, routes: AsyncRoutesResource) -> None:
        self._routes = routes

        self.create = async_to_streamed_response_wrapper(
            routes.create,
        )
        self.update = async_to_streamed_response_wrapper(
            routes.update,
        )
        self.list = async_to_streamed_response_wrapper(
            routes.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            routes.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            routes.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.zero_trust.gateway import pacfile_create_params, pacfile_update_params
from ....types.zero_trust.gateway.pacfile_get_response import PacfileGetResponse
from ....types.zero_trust.gateway.pacfile_list_response import PacfileListResponse
from ....types.zero_trust.gateway.pacfile_create_response import PacfileCreateResponse
from ....types.zero_trust.gateway.pacfile_update_response import PacfileUpdateResponse

__all__ = ["PacfilesResource", "AsyncPacfilesResource"]


class PacfilesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PacfilesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return PacfilesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PacfilesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return PacfilesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        contents: str,
        name: str,
        description: str | Omit = omit,
        slug: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PacfileCreateResponse]:
        """
        Create a new Zero Trust Gateway PAC file.

        Args:
          contents: Actual contents of the PAC file

          name: Name of the PAC file.

          description: Detailed description of the PAC file.

          slug: URL-friendly version of the PAC file name. If not provided, it will be
              auto-generated

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/gateway/pacfiles", account_id=account_id),
            body=maybe_transform(
                {
                    "contents": contents,
                    "name": name,
                    "description": description,
                    "slug": slug,
                },
                pacfile_create_params.PacfileCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PacfileCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PacfileCreateResponse]], ResultWrapper[PacfileCreateResponse]),
        )

    def update(
        self,
        pacfile_id: str,
        *,
        account_id: str,
        contents: str,
        description: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PacfileUpdateResponse]:
        """
        Update a configured Zero Trust Gateway PAC file.

        Args:
          contents: Actual contents of the PAC file

          description: Detailed description of the PAC file.

          name: Name of the PAC file.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not pacfile_id:
            raise ValueError(f"Expected a non-empty value for `pacfile_id` but received {pacfile_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/gateway/pacfiles/{pacfile_id}", account_id=account_id, pacfile_id=pacfile_id
            ),
            body=maybe_transform(
                {
                    "contents": contents,
                    "description": description,
                    "name": name,
                },
                pacfile_update_params.PacfileUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PacfileUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PacfileUpdateResponse]], ResultWrapper[PacfileUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[PacfileListResponse]:
        """
        List all Zero Trust Gateway PAC files for an account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/gateway/pacfiles", account_id=account_id),
            page=SyncSinglePage[PacfileListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=PacfileListResponse,
        )

    def delete(
        self,
        pacfile_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete a configured Zero Trust Gateway PAC file.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not pacfile_id:
            raise ValueError(f"Expected a non-empty value for `pacfile_id` but received {pacfile_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/gateway/pacfiles/{pacfile_id}", account_id=account_id, pacfile_id=pacfile_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def get(
        self,
        pacfile_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PacfileGetResponse]:
        """
        Get a single Zero Trust Gateway PAC file.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not pacfile_id:
            raise ValueError(f"Expected a non-empty value for `pacfile_id` but received {pacfile_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/gateway/pacfiles/{pacfile_id}", account_id=account_id, pacfile_id=pacfile_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PacfileGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PacfileGetResponse]], ResultWrapper[PacfileGetResponse]),
        )


class AsyncPacfilesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPacfilesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPacfilesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPacfilesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncPacfilesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        contents: str,
        name: str,
        description: str | Omit = omit,
        slug: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PacfileCreateResponse]:
        """
        Create a new Zero Trust Gateway PAC file.

        Args:
          contents: Actual contents of the PAC file

          name: Name of the PAC file.

          description: Detailed description of the PAC file.

          slug: URL-friendly version of the PAC file name. If not provided, it will be
              auto-generated

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/gateway/pacfiles", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "contents": contents,
                    "name": name,
                    "description": description,
                    "slug": slug,
                },
                pacfile_create_params.PacfileCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PacfileCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PacfileCreateResponse]], ResultWrapper[PacfileCreateResponse]),
        )

    async def update(
        self,
        pacfile_id: str,
        *,
        account_id: str,
        contents: str,
        description: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PacfileUpdateResponse]:
        """
        Update a configured Zero Trust Gateway PAC file.

        Args:
          contents: Actual contents of the PAC file

          description: Detailed description of the PAC file.

          name: Name of the PAC file.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not pacfile_id:
            raise ValueError(f"Expected a non-empty value for `pacfile_id` but received {pacfile_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/gateway/pacfiles/{pacfile_id}", account_id=account_id, pacfile_id=pacfile_id
            ),
            body=await async_maybe_transform(
                {
                    "contents": contents,
                    "description": description,
                    "name": name,
                },
                pacfile_update_params.PacfileUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PacfileUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PacfileUpdateResponse]], ResultWrapper[PacfileUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PacfileListResponse, AsyncSinglePage[PacfileListResponse]]:
        """
        List all Zero Trust Gateway PAC files for an account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/gateway/pacfiles", account_id=account_id),
            page=AsyncSinglePage[PacfileListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=PacfileListResponse,
        )

    async def delete(
        self,
        pacfile_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete a configured Zero Trust Gateway PAC file.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not pacfile_id:
            raise ValueError(f"Expected a non-empty value for `pacfile_id` but received {pacfile_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/gateway/pacfiles/{pacfile_id}", account_id=account_id, pacfile_id=pacfile_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def get(
        self,
        pacfile_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PacfileGetResponse]:
        """
        Get a single Zero Trust Gateway PAC file.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not pacfile_id:
            raise ValueError(f"Expected a non-empty value for `pacfile_id` but received {pacfile_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/gateway/pacfiles/{pacfile_id}", account_id=account_id, pacfile_id=pacfile_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PacfileGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PacfileGetResponse]], ResultWrapper[PacfileGetResponse]),
        )


class PacfilesResourceWithRawResponse:
    def __init__(self, pacfiles: PacfilesResource) -> None:
        self._pacfiles = pacfiles

        self.create = to_raw_response_wrapper(
            pacfiles.create,
        )
        self.update = to_raw_response_wrapper(
            pacfiles.update,
        )
        self.list = to_raw_response_wrapper(
            pacfiles.list,
        )
        self.delete = to_raw_response_wrapper(
            pacfiles.delete,
        )
        self.get = to_raw_response_wrapper(
            pacfiles.get,
        )


class AsyncPacfilesResourceWithRawResponse:
    def __init__(self, pacfiles: AsyncPacfilesResource) -> None:
        self._pacfiles = pacfiles

        self.create = async_to_raw_response_wrapper(
            pacfiles.create,
        )
        self.update = async_to_raw_response_wrapper(
            pacfiles.update,
        )
        self.list = async_to_raw_response_wrapper(
            pacfiles.list,
        )
        self.delete = async_to_raw_response_wrapper(
            pacfiles.delete,
        )
        self.get = async_to_raw_response_wrapper(
            pacfiles.get,
        )


class PacfilesResourceWithStreamingResponse:
    def __init__(self, pacfiles: PacfilesResource) -> None:
        self._pacfiles = pacfiles

        self.create = to_streamed_response_wrapper(
            pacfiles.create,
        )
        self.update = to_streamed_response_wrapper(
            pacfiles.update,
        )
        self.list = to_streamed_response_wrapper(
            pacfiles.list,
        )
        self.delete = to_streamed_response_wrapper(
            pacfiles.delete,
        )
        self.get = to_streamed_response_wrapper(
            pacfiles.get,
        )


class AsyncPacfilesResourceWithStreamingResponse:
    def __init__(self, pacfiles: AsyncPacfilesResource) -> None:
        self._pacfiles = pacfiles

        self.create = async_to_streamed_response_wrapper(
            pacfiles.create,
        )
        self.update = async_to_streamed_response_wrapper(
            pacfiles.update,
        )
        self.list = async_to_streamed_response_wrapper(
            pacfiles.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            pacfiles.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            pacfiles.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from .topup.topup import (
    TopupResource,
    AsyncTopupResource,
    TopupResourceWithRawResponse,
    AsyncTopupResourceWithRawResponse,
    TopupResourceWithStreamingResponse,
    AsyncTopupResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from .spending_limit import (
    SpendingLimitResource,
    AsyncSpendingLimitResource,
    SpendingLimitResourceWithRawResponse,
    AsyncSpendingLimitResourceWithRawResponse,
    SpendingLimitResourceWithStreamingResponse,
    AsyncSpendingLimitResourceWithStreamingResponse,
)
from ...._base_client import make_request_options
from ....types.ai_gateway import billing_usage_history_params, billing_invoice_history_params
from ....types.ai_gateway.billing_usage_history_response import BillingUsageHistoryResponse
from ....types.ai_gateway.billing_credit_balance_response import BillingCreditBalanceResponse
from ....types.ai_gateway.billing_invoice_history_response import BillingInvoiceHistoryResponse
from ....types.ai_gateway.billing_invoice_preview_response import BillingInvoicePreviewResponse

__all__ = ["BillingResource", "AsyncBillingResource"]


class BillingResource(SyncAPIResource):
    @cached_property
    def topup(self) -> TopupResource:
        return TopupResource(self._client)

    @cached_property
    def spending_limit(self) -> SpendingLimitResource:
        return SpendingLimitResource(self._client)

    @cached_property
    def with_raw_response(self) -> BillingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return BillingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BillingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return BillingResourceWithStreamingResponse(self)

    def credit_balance(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BillingCreditBalanceResponse:
        """
        Retrieve the current credit balance, payment method info, and top-up
        configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/ai-gateway/billing/credit-balance", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[BillingCreditBalanceResponse]._unwrapper,
            ),
            cast_to=cast(Type[BillingCreditBalanceResponse], ResultWrapper[BillingCreditBalanceResponse]),
        )

    def invoice_history(
        self,
        *,
        account_id: str,
        type: Literal["auto", "all", "manual"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BillingInvoiceHistoryResponse:
        """
        Retrieve a list of past invoices with pagination, optionally filtered by type.

        Args:
          type: Filter invoice type: auto, manual, or all.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/ai-gateway/billing/invoice-history", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"type": type}, billing_invoice_history_params.BillingInvoiceHistoryParams),
                post_parser=ResultWrapper[BillingInvoiceHistoryResponse]._unwrapper,
            ),
            cast_to=cast(Type[BillingInvoiceHistoryResponse], ResultWrapper[BillingInvoiceHistoryResponse]),
        )

    def invoice_preview(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BillingInvoicePreviewResponse:
        """
        Retrieve a preview of the upcoming invoice including line items and tax.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/ai-gateway/billing/invoice-preview", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[BillingInvoicePreviewResponse]._unwrapper,
            ),
            cast_to=cast(Type[BillingInvoicePreviewResponse], ResultWrapper[BillingInvoicePreviewResponse]),
        )

    def usage_history(
        self,
        *,
        account_id: str,
        value_grouping_window: Literal["day", "hour"],
        end_time: Optional[float] | Omit = omit,
        start_time: Optional[float] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BillingUsageHistoryResponse:
        """
        Retrieve aggregated usage meter event summaries for the given time range.

        Args:
          value_grouping_window: Grouping window for usage data.

          end_time: End time as Unix timestamp in milliseconds.

          start_time: Start time as Unix timestamp in milliseconds.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/ai-gateway/billing/usage-history", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "value_grouping_window": value_grouping_window,
                        "end_time": end_time,
                        "start_time": start_time,
                    },
                    billing_usage_history_params.BillingUsageHistoryParams,
                ),
                post_parser=ResultWrapper[BillingUsageHistoryResponse]._unwrapper,
            ),
            cast_to=cast(Type[BillingUsageHistoryResponse], ResultWrapper[BillingUsageHistoryResponse]),
        )


class AsyncBillingResource(AsyncAPIResource):
    @cached_property
    def topup(self) -> AsyncTopupResource:
        return AsyncTopupResource(self._client)

    @cached_property
    def spending_limit(self) -> AsyncSpendingLimitResource:
        return AsyncSpendingLimitResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncBillingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBillingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBillingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncBillingResourceWithStreamingResponse(self)

    async def credit_balance(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BillingCreditBalanceResponse:
        """
        Retrieve the current credit balance, payment method info, and top-up
        configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/ai-gateway/billing/credit-balance", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[BillingCreditBalanceResponse]._unwrapper,
            ),
            cast_to=cast(Type[BillingCreditBalanceResponse], ResultWrapper[BillingCreditBalanceResponse]),
        )

    async def invoice_history(
        self,
        *,
        account_id: str,
        type: Literal["auto", "all", "manual"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BillingInvoiceHistoryResponse:
        """
        Retrieve a list of past invoices with pagination, optionally filtered by type.

        Args:
          type: Filter invoice type: auto, manual, or all.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/ai-gateway/billing/invoice-history", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"type": type}, billing_invoice_history_params.BillingInvoiceHistoryParams
                ),
                post_parser=ResultWrapper[BillingInvoiceHistoryResponse]._unwrapper,
            ),
            cast_to=cast(Type[BillingInvoiceHistoryResponse], ResultWrapper[BillingInvoiceHistoryResponse]),
        )

    async def invoice_preview(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BillingInvoicePreviewResponse:
        """
        Retrieve a preview of the upcoming invoice including line items and tax.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/ai-gateway/billing/invoice-preview", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[BillingInvoicePreviewResponse]._unwrapper,
            ),
            cast_to=cast(Type[BillingInvoicePreviewResponse], ResultWrapper[BillingInvoicePreviewResponse]),
        )

    async def usage_history(
        self,
        *,
        account_id: str,
        value_grouping_window: Literal["day", "hour"],
        end_time: Optional[float] | Omit = omit,
        start_time: Optional[float] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BillingUsageHistoryResponse:
        """
        Retrieve aggregated usage meter event summaries for the given time range.

        Args:
          value_grouping_window: Grouping window for usage data.

          end_time: End time as Unix timestamp in milliseconds.

          start_time: Start time as Unix timestamp in milliseconds.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/ai-gateway/billing/usage-history", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "value_grouping_window": value_grouping_window,
                        "end_time": end_time,
                        "start_time": start_time,
                    },
                    billing_usage_history_params.BillingUsageHistoryParams,
                ),
                post_parser=ResultWrapper[BillingUsageHistoryResponse]._unwrapper,
            ),
            cast_to=cast(Type[BillingUsageHistoryResponse], ResultWrapper[BillingUsageHistoryResponse]),
        )


class BillingResourceWithRawResponse:
    def __init__(self, billing: BillingResource) -> None:
        self._billing = billing

        self.credit_balance = to_raw_response_wrapper(
            billing.credit_balance,
        )
        self.invoice_history = to_raw_response_wrapper(
            billing.invoice_history,
        )
        self.invoice_preview = to_raw_response_wrapper(
            billing.invoice_preview,
        )
        self.usage_history = to_raw_response_wrapper(
            billing.usage_history,
        )

    @cached_property
    def topup(self) -> TopupResourceWithRawResponse:
        return TopupResourceWithRawResponse(self._billing.topup)

    @cached_property
    def spending_limit(self) -> SpendingLimitResourceWithRawResponse:
        return SpendingLimitResourceWithRawResponse(self._billing.spending_limit)


class AsyncBillingResourceWithRawResponse:
    def __init__(self, billing: AsyncBillingResource) -> None:
        self._billing = billing

        self.credit_balance = async_to_raw_response_wrapper(
            billing.credit_balance,
        )
        self.invoice_history = async_to_raw_response_wrapper(
            billing.invoice_history,
        )
        self.invoice_preview = async_to_raw_response_wrapper(
            billing.invoice_preview,
        )
        self.usage_history = async_to_raw_response_wrapper(
            billing.usage_history,
        )

    @cached_property
    def topup(self) -> AsyncTopupResourceWithRawResponse:
        return AsyncTopupResourceWithRawResponse(self._billing.topup)

    @cached_property
    def spending_limit(self) -> AsyncSpendingLimitResourceWithRawResponse:
        return AsyncSpendingLimitResourceWithRawResponse(self._billing.spending_limit)


class BillingResourceWithStreamingResponse:
    def __init__(self, billing: BillingResource) -> None:
        self._billing = billing

        self.credit_balance = to_streamed_response_wrapper(
            billing.credit_balance,
        )
        self.invoice_history = to_streamed_response_wrapper(
            billing.invoice_history,
        )
        self.invoice_preview = to_streamed_response_wrapper(
            billing.invoice_preview,
        )
        self.usage_history = to_streamed_response_wrapper(
            billing.usage_history,
        )

    @cached_property
    def topup(self) -> TopupResourceWithStreamingResponse:
        return TopupResourceWithStreamingResponse(self._billing.topup)

    @cached_property
    def spending_limit(self) -> SpendingLimitResourceWithStreamingResponse:
        return SpendingLimitResourceWithStreamingResponse(self._billing.spending_limit)


class AsyncBillingResourceWithStreamingResponse:
    def __init__(self, billing: AsyncBillingResource) -> None:
        self._billing = billing

        self.credit_balance = async_to_streamed_response_wrapper(
            billing.credit_balance,
        )
        self.invoice_history = async_to_streamed_response_wrapper(
            billing.invoice_history,
        )
        self.invoice_preview = async_to_streamed_response_wrapper(
            billing.invoice_preview,
        )
        self.usage_history = async_to_streamed_response_wrapper(
            billing.usage_history,
        )

    @cached_property
    def topup(self) -> AsyncTopupResourceWithStreamingResponse:
        return AsyncTopupResourceWithStreamingResponse(self._billing.topup)

    @cached_property
    def spending_limit(self) -> AsyncSpendingLimitResourceWithStreamingResponse:
        return AsyncSpendingLimitResourceWithStreamingResponse(self._billing.spending_limit)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .rules.rules import (
    RulesResource,
    AsyncRulesResource,
    RulesResourceWithRawResponse,
    AsyncRulesResourceWithRawResponse,
    RulesResourceWithStreamingResponse,
    AsyncRulesResourceWithStreamingResponse,
)
from .configs.configs import (
    ConfigsResource,
    AsyncConfigsResource,
    ConfigsResourceWithRawResponse,
    AsyncConfigsResourceWithRawResponse,
    ConfigsResourceWithStreamingResponse,
    AsyncConfigsResourceWithStreamingResponse,
)
from .vpc_flows.vpc_flows import (
    VPCFlowsResource,
    AsyncVPCFlowsResource,
    VPCFlowsResourceWithRawResponse,
    AsyncVPCFlowsResourceWithRawResponse,
    VPCFlowsResourceWithStreamingResponse,
    AsyncVPCFlowsResourceWithStreamingResponse,
)

__all__ = ["MagicNetworkMonitoringResource", "AsyncMagicNetworkMonitoringResource"]


class MagicNetworkMonitoringResource(SyncAPIResource):
    @cached_property
    def vpc_flows(self) -> VPCFlowsResource:
        return VPCFlowsResource(self._client)

    @cached_property
    def configs(self) -> ConfigsResource:
        return ConfigsResource(self._client)

    @cached_property
    def rules(self) -> RulesResource:
        return RulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> MagicNetworkMonitoringResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return MagicNetworkMonitoringResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MagicNetworkMonitoringResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return MagicNetworkMonitoringResourceWithStreamingResponse(self)


class AsyncMagicNetworkMonitoringResource(AsyncAPIResource):
    @cached_property
    def vpc_flows(self) -> AsyncVPCFlowsResource:
        return AsyncVPCFlowsResource(self._client)

    @cached_property
    def configs(self) -> AsyncConfigsResource:
        return AsyncConfigsResource(self._client)

    @cached_property
    def rules(self) -> AsyncRulesResource:
        return AsyncRulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncMagicNetworkMonitoringResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMagicNetworkMonitoringResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMagicNetworkMonitoringResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncMagicNetworkMonitoringResourceWithStreamingResponse(self)


class MagicNetworkMonitoringResourceWithRawResponse:
    def __init__(self, magic_network_monitoring: MagicNetworkMonitoringResource) -> None:
        self._magic_network_monitoring = magic_network_monitoring

    @cached_property
    def vpc_flows(self) -> VPCFlowsResourceWithRawResponse:
        return VPCFlowsResourceWithRawResponse(self._magic_network_monitoring.vpc_flows)

    @cached_property
    def configs(self) -> ConfigsResourceWithRawResponse:
        return ConfigsResourceWithRawResponse(self._magic_network_monitoring.configs)

    @cached_property
    def rules(self) -> RulesResourceWithRawResponse:
        return RulesResourceWithRawResponse(self._magic_network_monitoring.rules)


class AsyncMagicNetworkMonitoringResourceWithRawResponse:
    def __init__(self, magic_network_monitoring: AsyncMagicNetworkMonitoringResource) -> None:
        self._magic_network_monitoring = magic_network_monitoring

    @cached_property
    def vpc_flows(self) -> AsyncVPCFlowsResourceWithRawResponse:
        return AsyncVPCFlowsResourceWithRawResponse(self._magic_network_monitoring.vpc_flows)

    @cached_property
    def configs(self) -> AsyncConfigsResourceWithRawResponse:
        return AsyncConfigsResourceWithRawResponse(self._magic_network_monitoring.configs)

    @cached_property
    def rules(self) -> AsyncRulesResourceWithRawResponse:
        return AsyncRulesResourceWithRawResponse(self._magic_network_monitoring.rules)


class MagicNetworkMonitoringResourceWithStreamingResponse:
    def __init__(self, magic_network_monitoring: MagicNetworkMonitoringResource) -> None:
        self._magic_network_monitoring = magic_network_monitoring

    @cached_property
    def vpc_flows(self) -> VPCFlowsResourceWithStreamingResponse:
        return VPCFlowsResourceWithStreamingResponse(self._magic_network_monitoring.vpc_flows)

    @cached_property
    def configs(self) -> ConfigsResourceWithStreamingResponse:
        return ConfigsResourceWithStreamingResponse(self._magic_network_monitoring.configs)

    @cached_property
    def rules(self) -> RulesResourceWithStreamingResponse:
        return RulesResourceWithStreamingResponse(self._magic_network_monitoring.rules)


class AsyncMagicNetworkMonitoringResourceWithStreamingResponse:
    def __init__(self, magic_network_monitoring: AsyncMagicNetworkMonitoringResource) -> None:
        self._magic_network_monitoring = magic_network_monitoring

    @cached_property
    def vpc_flows(self) -> AsyncVPCFlowsResourceWithStreamingResponse:
        return AsyncVPCFlowsResourceWithStreamingResponse(self._magic_network_monitoring.vpc_flows)

    @cached_property
    def configs(self) -> AsyncConfigsResourceWithStreamingResponse:
        return AsyncConfigsResourceWithStreamingResponse(self._magic_network_monitoring.configs)

    @cached_property
    def rules(self) -> AsyncRulesResourceWithStreamingResponse:
        return AsyncRulesResourceWithStreamingResponse(self._magic_network_monitoring.rules)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .label import Label
from ...._types import SequenceNotStr

__all__ = ["PriorityCreateParams"]


class PriorityCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    labels: Required[SequenceNotStr[Label]]
    """List of labels."""

    priority: Required[int]
    """Priority."""

    requirement: Required[str]
    """Requirement."""

    tlp: Required[Literal["clear", "amber", "amber-strict", "green", "red"]]
    """The CISA defined Traffic Light Protocol (TLP)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = [
    "ConfigUpdateParams",
    "Origin",
    "OriginPublicDatabase",
    "OriginAccessProtectedDatabaseBehindCloudflareTunnel",
    "OriginDatabaseReachableThroughAWorkersVPC",
    "Caching",
    "CachingHyperdriveHyperdriveCachingCommon",
    "CachingHyperdriveHyperdriveCachingEnabled",
    "MTLS",
]


class ConfigUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Define configurations using a unique string identifier."""

    name: Required[str]
    """The name of the Hyperdrive configuration.

    Used to identify the configuration in the Cloudflare dashboard and API.
    """

    origin: Required[Origin]

    caching: Caching

    mtls: MTLS
    """mTLS configuration for the origin connection.

    Cannot be used with VPC Service origins; TLS must be managed on the VPC Service.
    """

    origin_connection_limit: int
    """
    The (soft) maximum number of connections the Hyperdrive is allowed to make to
    the origin database.

    Maximum allowed: 20 for free tier accounts, 100 for paid tier accounts. If not
    specified, defaults to 20 for free tier and 60 for paid tier. Contact Cloudflare
    if you need a higher limit.
    """


class OriginPublicDatabase(TypedDict, total=False):
    database: Required[str]
    """Set the name of your origin database."""

    host: Required[str]
    """Defines the host (hostname or IP) of your origin database."""

    password: Required[str]
    """Set the password needed to access your origin database.

    The API never returns this write-only value.
    """

    port: Required[int]
    """Defines the port of your origin database.

    Defaults to 5432 for PostgreSQL or 3306 for MySQL if not specified.
    """

    scheme: Required[Literal["postgres", "postgresql", "mysql"]]
    """Specifies the URL scheme used to connect to your origin database."""

    user: Required[str]
    """Set the user of your origin database."""


class OriginAccessProtectedDatabaseBehindCloudflareTunnel(TypedDict, total=False):
    access_client_id: Required[str]
    """
    Defines the Client ID of the Access token to use when connecting to the origin
    database.
    """

    access_client_secret: Required[str]
    """
    Defines the Client Secret of the Access Token to use when connecting to the
    origin database. The API never returns this write-only value.
    """

    database: Required[str]
    """Set the name of your origin database."""

    host: Required[str]
    """Defines the host (hostname or IP) of your origin database."""

    password: Required[str]
    """Set the password needed to access your origin database.

    The API never returns this write-only value.
    """

    scheme: Required[Literal["postgres", "postgresql", "mysql"]]
    """Specifies the URL scheme used to connect to your origin database."""

    user: Required[str]
    """Set the user of your origin database."""


class OriginDatabaseReachableThroughAWorkersVPC(TypedDict, total=False):
    database: Required[str]
    """Set the name of your origin database."""

    password: Required[str]
    """Set the password needed to access your origin database.

    The API never returns this write-only value.
    """

    scheme: Required[Literal["postgres", "postgresql", "mysql"]]
    """Specifies the URL scheme used to connect to your origin database."""

    service_id: Required[str]
    """The identifier of the Workers VPC Service to connect through.

    Hyperdrive will egress through the specified VPC Service to reach the origin
    database.
    """

    user: Required[str]
    """Set the user of your origin database."""


Origin: TypeAlias = Union[
    OriginPublicDatabase, OriginAccessProtectedDatabaseBehindCloudflareTunnel, OriginDatabaseReachableThroughAWorkersVPC
]


class CachingHyperdriveHyperdriveCachingCommon(TypedDict, total=False):
    disabled: bool
    """Set to true to disable caching of SQL responses. Default is false."""


class CachingHyperdriveHyperdriveCachingEnabled(TypedDict, total=False):
    disabled: bool
    """Set to true to disable caching of SQL responses. Default is false."""

    max_age: int
    """Specify the maximum duration (in seconds) items should persist in the cache.

    Defaults to 60 seconds if not specified.
    """

    stale_while_revalidate: int
    """Specify the number of seconds the cache may serve a stale response.

    Defaults to 15 seconds if not specified.
    """


Caching: TypeAlias = Union[CachingHyperdriveHyperdriveCachingCommon, CachingHyperdriveHyperdriveCachingEnabled]


class MTLS(TypedDict, total=False):
    """mTLS configuration for the origin connection.

    Cannot be used with VPC Service origins; TLS must be managed on the VPC Service.
    """

    ca_certificate_id: str
    """Define CA certificate ID obtained after uploading CA cert."""

    mtls_certificate_id: str
    """Define mTLS certificate ID obtained after uploading client cert."""

    sslmode: str
    """Set SSL mode to 'require', 'verify-ca', or 'verify-full' to verify the CA."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = [
    "BillingInvoicePreviewResponse",
    "InvoiceLine",
    "InvoiceLinePeriod",
    "InvoiceLinePricing",
    "InvoiceLinePretaxCreditAmount",
]


class InvoiceLinePeriod(BaseModel):
    end: float

    start: float


class InvoiceLinePricing(BaseModel):
    unit_amount_decimal: Optional[str] = None


class InvoiceLinePretaxCreditAmount(BaseModel):
    amount: float

    type: str

    credit_balance_transaction: Optional[str] = None

    discount: Optional[str] = None


class InvoiceLine(BaseModel):
    amount: float

    currency: str

    description: Optional[str] = None

    period: InvoiceLinePeriod

    pricing: InvoiceLinePricing

    quantity: float

    pretax_credit_amounts: Optional[List[InvoiceLinePretaxCreditAmount]] = None


class BillingInvoicePreviewResponse(BaseModel):
    id: str

    amount_due: float

    amount_paid: float

    amount_remaining: float

    currency: str

    invoice_lines: List[InvoiceLine]

    period_end: float

    period_start: float

    status: Literal["draft", "open", "paid", "uncollectible", "void"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["FallthroughCreateResponse"]


class FallthroughCreateResponse(BaseModel):
    expression: str
    """WAF Expression for fallthrough"""

    title: str
    """Title for the expression"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from ...._models import BaseModel

__all__ = ["OverrideCodeGetResponse"]


class OverrideCodeGetResponse(BaseModel):
    disable_for_time: Optional[Dict[str, str]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .logs import (
    LogsResource,
    AsyncLogsResource,
    LogsResourceWithRawResponse,
    AsyncLogsResourceWithRawResponse,
    LogsResourceWithStreamingResponse,
    AsyncLogsResourceWithStreamingResponse,
)
from .scim import (
    SCIMResource,
    AsyncSCIMResource,
    SCIMResourceWithRawResponse,
    AsyncSCIMResourceWithRawResponse,
    SCIMResourceWithStreamingResponse,
    AsyncSCIMResourceWithStreamingResponse,
)
from .access_requests import (
    AccessRequestsResource,
    AsyncAccessRequestsResource,
    AccessRequestsResourceWithRawResponse,
    AsyncAccessRequestsResourceWithRawResponse,
    AccessRequestsResourceWithStreamingResponse,
    AsyncAccessRequestsResourceWithStreamingResponse,
)

__all__ = [
    "AccessRequestsResource",
    "AsyncAccessRequestsResource",
    "AccessRequestsResourceWithRawResponse",
    "AsyncAccessRequestsResourceWithRawResponse",
    "AccessRequestsResourceWithStreamingResponse",
    "AsyncAccessRequestsResourceWithStreamingResponse",
    "SCIMResource",
    "AsyncSCIMResource",
    "SCIMResourceWithRawResponse",
    "AsyncSCIMResourceWithRawResponse",
    "SCIMResourceWithStreamingResponse",
    "AsyncSCIMResourceWithStreamingResponse",
    "LogsResource",
    "AsyncLogsResource",
    "LogsResourceWithRawResponse",
    "AsyncLogsResourceWithRawResponse",
    "LogsResourceWithStreamingResponse",
    "AsyncLogsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from ..._utils import PropertyInfo

__all__ = [
    "AccountTagUpdateParams",
    "ResourceTaggingSetTagsRequestAccountLevelWorkerVersion",
    "ResourceTaggingSetTagsRequestAccountLevelBase",
]


class ResourceTaggingSetTagsRequestAccountLevelWorkerVersion(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    resource_id: Required[str]
    """Identifies the unique resource."""

    resource_type: Required[
        Literal[
            "access_application",
            "access_group",
            "account",
            "ai_gateway",
            "alerting_policy",
            "alerting_webhook",
            "cloudflared_tunnel",
            "d1_database",
            "durable_object_namespace",
            "gateway_list",
            "gateway_rule",
            "image",
            "kv_namespace",
            "queue",
            "r2_bucket",
            "resource_share",
            "stream_live_input",
            "stream_video",
            "worker",
            "worker_version",
        ]
    ]
    """
    Enum for base account-level resource types (those with no extra required
    fields).
    """

    worker_id: Required[str]
    """Worker ID is required only for worker_version resources"""

    tags: Dict[str, str]
    """Contains key-value pairs of tags."""

    if_match: Annotated[str, PropertyInfo(alias="If-Match")]


class ResourceTaggingSetTagsRequestAccountLevelBase(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    resource_id: Required[str]
    """Identifies the unique resource."""

    resource_type: Required[
        Literal[
            "access_application",
            "access_group",
            "account",
            "ai_gateway",
            "alerting_policy",
            "alerting_webhook",
            "cloudflared_tunnel",
            "d1_database",
            "durable_object_namespace",
            "gateway_list",
            "gateway_rule",
            "image",
            "kv_namespace",
            "queue",
            "r2_bucket",
            "resource_share",
            "stream_live_input",
            "stream_video",
            "worker",
        ]
    ]
    """
    Enum for base account-level resource types (those with no extra required
    fields).
    """

    tags: Dict[str, str]
    """Contains key-value pairs of tags."""

    if_match: Annotated[str, PropertyInfo(alias="If-Match")]


AccountTagUpdateParams: TypeAlias = Union[
    ResourceTaggingSetTagsRequestAccountLevelWorkerVersion, ResourceTaggingSetTagsRequestAccountLevelBase
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from .routes.routes import (
    RoutesResource,
    AsyncRoutesResource,
    RoutesResourceWithRawResponse,
    AsyncRoutesResourceWithRawResponse,
    RoutesResourceWithStreamingResponse,
    AsyncRoutesResourceWithStreamingResponse,
)
from .hostname_routes import (
    HostnameRoutesResource,
    AsyncHostnameRoutesResource,
    HostnameRoutesResourceWithRawResponse,
    AsyncHostnameRoutesResourceWithRawResponse,
    HostnameRoutesResourceWithStreamingResponse,
    AsyncHostnameRoutesResourceWithStreamingResponse,
)
from .subnets.subnets import (
    SubnetsResource,
    AsyncSubnetsResource,
    SubnetsResourceWithRawResponse,
    AsyncSubnetsResourceWithRawResponse,
    SubnetsResourceWithStreamingResponse,
    AsyncSubnetsResourceWithStreamingResponse,
)
from .virtual_networks import (
    VirtualNetworksResource,
    AsyncVirtualNetworksResource,
    VirtualNetworksResourceWithRawResponse,
    AsyncVirtualNetworksResourceWithRawResponse,
    VirtualNetworksResourceWithStreamingResponse,
    AsyncVirtualNetworksResourceWithStreamingResponse,
)

__all__ = ["NetworksResource", "AsyncNetworksResource"]


class NetworksResource(SyncAPIResource):
    @cached_property
    def routes(self) -> RoutesResource:
        return RoutesResource(self._client)

    @cached_property
    def virtual_networks(self) -> VirtualNetworksResource:
        return VirtualNetworksResource(self._client)

    @cached_property
    def subnets(self) -> SubnetsResource:
        return SubnetsResource(self._client)

    @cached_property
    def hostname_routes(self) -> HostnameRoutesResource:
        return HostnameRoutesResource(self._client)

    @cached_property
    def with_raw_response(self) -> NetworksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return NetworksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> NetworksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return NetworksResourceWithStreamingResponse(self)


class AsyncNetworksResource(AsyncAPIResource):
    @cached_property
    def routes(self) -> AsyncRoutesResource:
        return AsyncRoutesResource(self._client)

    @cached_property
    def virtual_networks(self) -> AsyncVirtualNetworksResource:
        return AsyncVirtualNetworksResource(self._client)

    @cached_property
    def subnets(self) -> AsyncSubnetsResource:
        return AsyncSubnetsResource(self._client)

    @cached_property
    def hostname_routes(self) -> AsyncHostnameRoutesResource:
        return AsyncHostnameRoutesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncNetworksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncNetworksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncNetworksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncNetworksResourceWithStreamingResponse(self)


class NetworksResourceWithRawResponse:
    def __init__(self, networks: NetworksResource) -> None:
        self._networks = networks

    @cached_property
    def routes(self) -> RoutesResourceWithRawResponse:
        return RoutesResourceWithRawResponse(self._networks.routes)

    @cached_property
    def virtual_networks(self) -> VirtualNetworksResourceWithRawResponse:
        return VirtualNetworksResourceWithRawResponse(self._networks.virtual_networks)

    @cached_property
    def subnets(self) -> SubnetsResourceWithRawResponse:
        return SubnetsResourceWithRawResponse(self._networks.subnets)

    @cached_property
    def hostname_routes(self) -> HostnameRoutesResourceWithRawResponse:
        return HostnameRoutesResourceWithRawResponse(self._networks.hostname_routes)


class AsyncNetworksResourceWithRawResponse:
    def __init__(self, networks: AsyncNetworksResource) -> None:
        self._networks = networks

    @cached_property
    def routes(self) -> AsyncRoutesResourceWithRawResponse:
        return AsyncRoutesResourceWithRawResponse(self._networks.routes)

    @cached_property
    def virtual_networks(self) -> AsyncVirtualNetworksResourceWithRawResponse:
        return AsyncVirtualNetworksResourceWithRawResponse(self._networks.virtual_networks)

    @cached_property
    def subnets(self) -> AsyncSubnetsResourceWithRawResponse:
        return AsyncSubnetsResourceWithRawResponse(self._networks.subnets)

    @cached_property
    def hostname_routes(self) -> AsyncHostnameRoutesResourceWithRawResponse:
        return AsyncHostnameRoutesResourceWithRawResponse(self._networks.hostname_routes)


class NetworksResourceWithStreamingResponse:
    def __init__(self, networks: NetworksResource) -> None:
        self._networks = networks

    @cached_property
    def routes(self) -> RoutesResourceWithStreamingResponse:
        return RoutesResourceWithStreamingResponse(self._networks.routes)

    @cached_property
    def virtual_networks(self) -> VirtualNetworksResourceWithStreamingResponse:
        return VirtualNetworksResourceWithStreamingResponse(self._networks.virtual_networks)

    @cached_property
    def subnets(self) -> SubnetsResourceWithStreamingResponse:
        return SubnetsResourceWithStreamingResponse(self._networks.subnets)

    @cached_property
    def hostname_routes(self) -> HostnameRoutesResourceWithStreamingResponse:
        return HostnameRoutesResourceWithStreamingResponse(self._networks.hostname_routes)


class AsyncNetworksResourceWithStreamingResponse:
    def __init__(self, networks: AsyncNetworksResource) -> None:
        self._networks = networks

    @cached_property
    def routes(self) -> AsyncRoutesResourceWithStreamingResponse:
        return AsyncRoutesResourceWithStreamingResponse(self._networks.routes)

    @cached_property
    def virtual_networks(self) -> AsyncVirtualNetworksResourceWithStreamingResponse:
        return AsyncVirtualNetworksResourceWithStreamingResponse(self._networks.virtual_networks)

    @cached_property
    def subnets(self) -> AsyncSubnetsResourceWithStreamingResponse:
        return AsyncSubnetsResourceWithStreamingResponse(self._networks.subnets)

    @cached_property
    def hostname_routes(self) -> AsyncHostnameRoutesResourceWithStreamingResponse:
        return AsyncHostnameRoutesResourceWithStreamingResponse(self._networks.hostname_routes)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["ItemCreateResponse"]


class ItemCreateResponse(BaseModel):
    operation_id: str
    """The unique operation ID of the asynchronous action."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, Optional, cast
from typing_extensions import Literal, overload

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, required_args, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.queues import message_ack_params, message_pull_params, message_push_params, message_bulk_push_params
from ...types.queues.message_ack_response import MessageAckResponse
from ...types.queues.message_pull_response import MessagePullResponse
from ...types.queues.message_push_response import MessagePushResponse
from ...types.queues.message_bulk_push_response import MessageBulkPushResponse

__all__ = ["MessagesResource", "AsyncMessagesResource"]


class MessagesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MessagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return MessagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MessagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return MessagesResourceWithStreamingResponse(self)

    def ack(
        self,
        queue_id: str,
        *,
        account_id: str,
        acks: Iterable[message_ack_params.Ack] | Omit = omit,
        retries: Iterable[message_ack_params.Retry] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MessageAckResponse]:
        """
        Acknowledge + Retry messages from a Queue

        Args:
          account_id: A Resource identifier.

          queue_id: A Resource identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not queue_id:
            raise ValueError(f"Expected a non-empty value for `queue_id` but received {queue_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/queues/{queue_id}/messages/ack", account_id=account_id, queue_id=queue_id
            ),
            body=maybe_transform(
                {
                    "acks": acks,
                    "retries": retries,
                },
                message_ack_params.MessageAckParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MessageAckResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[MessageAckResponse]], ResultWrapper[MessageAckResponse]),
        )

    def bulk_push(
        self,
        queue_id: str,
        *,
        account_id: str,
        delay_seconds: float | Omit = omit,
        messages: Iterable[message_bulk_push_params.Message] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MessageBulkPushResponse]:
        """
        Push a batch of message to a Queue

        Args:
          account_id: A Resource identifier.

          queue_id: A Resource identifier.

          delay_seconds: The number of seconds to wait for attempting to deliver this batch to consumers

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not queue_id:
            raise ValueError(f"Expected a non-empty value for `queue_id` but received {queue_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/queues/{queue_id}/messages/batch", account_id=account_id, queue_id=queue_id
            ),
            body=maybe_transform(
                {
                    "delay_seconds": delay_seconds,
                    "messages": messages,
                },
                message_bulk_push_params.MessageBulkPushParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MessageBulkPushResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[MessageBulkPushResponse]], ResultWrapper[MessageBulkPushResponse]),
        )

    def pull(
        self,
        queue_id: str,
        *,
        account_id: str,
        batch_size: float | Omit = omit,
        visibility_timeout_ms: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MessagePullResponse]:
        """
        Pull a batch of messages from a Queue

        Args:
          account_id: A Resource identifier.

          queue_id: A Resource identifier.

          batch_size: The maximum number of messages to include in a batch.

          visibility_timeout_ms: The number of milliseconds that a message is exclusively leased. After the
              timeout, the message becomes available for another attempt.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not queue_id:
            raise ValueError(f"Expected a non-empty value for `queue_id` but received {queue_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/queues/{queue_id}/messages/pull", account_id=account_id, queue_id=queue_id
            ),
            body=maybe_transform(
                {
                    "batch_size": batch_size,
                    "visibility_timeout_ms": visibility_timeout_ms,
                },
                message_pull_params.MessagePullParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MessagePullResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[MessagePullResponse]], ResultWrapper[MessagePullResponse]),
        )

    @overload
    def push(
        self,
        queue_id: str,
        *,
        account_id: str,
        body: str | Omit = omit,
        content_type: Literal["text"] | Omit = omit,
        delay_seconds: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MessagePushResponse]:
        """
        Push a message to a Queue

        Args:
          account_id: A Resource identifier.

          queue_id: A Resource identifier.

          delay_seconds: The number of seconds to wait for attempting to deliver this message to
              consumers

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def push(
        self,
        queue_id: str,
        *,
        account_id: str,
        body: object | Omit = omit,
        content_type: Literal["json"] | Omit = omit,
        delay_seconds: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MessagePushResponse]:
        """
        Push a message to a Queue

        Args:
          account_id: A Resource identifier.

          queue_id: A Resource identifier.

          delay_seconds: The number of seconds to wait for attempting to deliver this message to
              consumers

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id"])
    def push(
        self,
        queue_id: str,
        *,
        account_id: str,
        body: str | object | Omit = omit,
        content_type: Literal["text"] | Literal["json"] | Omit = omit,
        delay_seconds: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MessagePushResponse]:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not queue_id:
            raise ValueError(f"Expected a non-empty value for `queue_id` but received {queue_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/queues/{queue_id}/messages", account_id=account_id, queue_id=queue_id
            ),
            body=maybe_transform(
                {
                    "body": body,
                    "content_type": content_type,
                    "delay_seconds": delay_seconds,
                },
                message_push_params.MessagePushParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MessagePushResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[MessagePushResponse]], ResultWrapper[MessagePushResponse]),
        )


class AsyncMessagesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMessagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMessagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMessagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncMessagesResourceWithStreamingResponse(self)

    async def ack(
        self,
        queue_id: str,
        *,
        account_id: str,
        acks: Iterable[message_ack_params.Ack] | Omit = omit,
        retries: Iterable[message_ack_params.Retry] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MessageAckResponse]:
        """
        Acknowledge + Retry messages from a Queue

        Args:
          account_id: A Resource identifier.

          queue_id: A Resource identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not queue_id:
            raise ValueError(f"Expected a non-empty value for `queue_id` but received {queue_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/queues/{queue_id}/messages/ack", account_id=account_id, queue_id=queue_id
            ),
            body=await async_maybe_transform(
                {
                    "acks": acks,
                    "retries": retries,
                },
                message_ack_params.MessageAckParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MessageAckResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[MessageAckResponse]], ResultWrapper[MessageAckResponse]),
        )

    async def bulk_push(
        self,
        queue_id: str,
        *,
        account_id: str,
        delay_seconds: float | Omit = omit,
        messages: Iterable[message_bulk_push_params.Message] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MessageBulkPushResponse]:
        """
        Push a batch of message to a Queue

        Args:
          account_id: A Resource identifier.

          queue_id: A Resource identifier.

          delay_seconds: The number of seconds to wait for attempting to deliver this batch to consumers

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not queue_id:
            raise ValueError(f"Expected a non-empty value for `queue_id` but received {queue_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/queues/{queue_id}/messages/batch", account_id=account_id, queue_id=queue_id
            ),
            body=await async_maybe_transform(
                {
                    "delay_seconds": delay_seconds,
                    "messages": messages,
                },
                message_bulk_push_params.MessageBulkPushParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MessageBulkPushResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[MessageBulkPushResponse]], ResultWrapper[MessageBulkPushResponse]),
        )

    async def pull(
        self,
        queue_id: str,
        *,
        account_id: str,
        batch_size: float | Omit = omit,
        visibility_timeout_ms: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MessagePullResponse]:
        """
        Pull a batch of messages from a Queue

        Args:
          account_id: A Resource identifier.

          queue_id: A Resource identifier.

          batch_size: The maximum number of messages to include in a batch.

          visibility_timeout_ms: The number of milliseconds that a message is exclusively leased. After the
              timeout, the message becomes available for another attempt.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not queue_id:
            raise ValueError(f"Expected a non-empty value for `queue_id` but received {queue_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/queues/{queue_id}/messages/pull", account_id=account_id, queue_id=queue_id
            ),
            body=await async_maybe_transform(
                {
                    "batch_size": batch_size,
                    "visibility_timeout_ms": visibility_timeout_ms,
                },
                message_pull_params.MessagePullParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MessagePullResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[MessagePullResponse]], ResultWrapper[MessagePullResponse]),
        )

    @overload
    async def push(
        self,
        queue_id: str,
        *,
        account_id: str,
        body: str | Omit = omit,
        content_type: Literal["text"] | Omit = omit,
        delay_seconds: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MessagePushResponse]:
        """
        Push a message to a Queue

        Args:
          account_id: A Resource identifier.

          queue_id: A Resource identifier.

          delay_seconds: The number of seconds to wait for attempting to deliver this message to
              consumers

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def push(
        self,
        queue_id: str,
        *,
        account_id: str,
        body: object | Omit = omit,
        content_type: Literal["json"] | Omit = omit,
        delay_seconds: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MessagePushResponse]:
        """
        Push a message to a Queue

        Args:
          account_id: A Resource identifier.

          queue_id: A Resource identifier.

          delay_seconds: The number of seconds to wait for attempting to deliver this message to
              consumers

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id"])
    async def push(
        self,
        queue_id: str,
        *,
        account_id: str,
        body: str | object | Omit = omit,
        content_type: Literal["text"] | Literal["json"] | Omit = omit,
        delay_seconds: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MessagePushResponse]:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not queue_id:
            raise ValueError(f"Expected a non-empty value for `queue_id` but received {queue_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/queues/{queue_id}/messages", account_id=account_id, queue_id=queue_id
            ),
            body=await async_maybe_transform(
                {
                    "body": body,
                    "content_type": content_type,
                    "delay_seconds": delay_seconds,
                },
                message_push_params.MessagePushParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MessagePushResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[MessagePushResponse]], ResultWrapper[MessagePushResponse]),
        )


class MessagesResourceWithRawResponse:
    def __init__(self, messages: MessagesResource) -> None:
        self._messages = messages

        self.ack = to_raw_response_wrapper(
            messages.ack,
        )
        self.bulk_push = to_raw_response_wrapper(
            messages.bulk_push,
        )
        self.pull = to_raw_response_wrapper(
            messages.pull,
        )
        self.push = to_raw_response_wrapper(
            messages.push,
        )


class AsyncMessagesResourceWithRawResponse:
    def __init__(self, messages: AsyncMessagesResource) -> None:
        self._messages = messages

        self.ack = async_to_raw_response_wrapper(
            messages.ack,
        )
        self.bulk_push = async_to_raw_response_wrapper(
            messages.bulk_push,
        )
        self.pull = async_to_raw_response_wrapper(
            messages.pull,
        )
        self.push = async_to_raw_response_wrapper(
            messages.push,
        )


class MessagesResourceWithStreamingResponse:
    def __init__(self, messages: MessagesResource) -> None:
        self._messages = messages

        self.ack = to_streamed_response_wrapper(
            messages.ack,
        )
        self.bulk_push = to_streamed_response_wrapper(
            messages.bulk_push,
        )
        self.pull = to_streamed_response_wrapper(
            messages.pull,
        )
        self.push = to_streamed_response_wrapper(
            messages.push,
        )


class AsyncMessagesResourceWithStreamingResponse:
    def __init__(self, messages: AsyncMessagesResource) -> None:
        self._messages = messages

        self.ack = async_to_streamed_response_wrapper(
            messages.ack,
        )
        self.bulk_push = async_to_streamed_response_wrapper(
            messages.bulk_push,
        )
        self.pull = async_to_streamed_response_wrapper(
            messages.pull,
        )
        self.push = async_to_streamed_response_wrapper(
            messages.push,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .scripts.consumer_script import ConsumerScript

__all__ = ["ScriptSetting", "Observability", "ObservabilityLogs", "ObservabilityTraces"]


class ObservabilityLogs(BaseModel):
    """Log settings for the Worker."""

    enabled: bool
    """Whether logs are enabled for the Worker."""

    invocation_logs: bool
    """
    Whether
    [invocation logs](https://developers.cloudflare.com/workers/observability/logs/workers-logs/#invocation-logs)
    are enabled for the Worker.
    """

    destinations: Optional[List[str]] = None
    """A list of destinations where logs will be exported to."""

    head_sampling_rate: Optional[float] = None
    """The sampling rate for logs. From 0 to 1 (1 = 100%, 0.1 = 10%). Default is 1."""

    persist: Optional[bool] = None
    """Whether log persistence is enabled for the Worker."""


class ObservabilityTraces(BaseModel):
    """Trace settings for the Worker."""

    destinations: Optional[List[str]] = None
    """A list of destinations where traces will be exported to."""

    enabled: Optional[bool] = None
    """Whether traces are enabled for the Worker."""

    head_sampling_rate: Optional[float] = None
    """The sampling rate for traces. From 0 to 1 (1 = 100%, 0.1 = 10%). Default is 1."""

    persist: Optional[bool] = None
    """Whether trace persistence is enabled for the Worker."""


class Observability(BaseModel):
    """Observability settings for the Worker."""

    enabled: bool
    """Whether observability is enabled for the Worker."""

    head_sampling_rate: Optional[float] = None
    """The sampling rate for incoming requests.

    From 0 to 1 (1 = 100%, 0.1 = 10%). Default is 1.
    """

    logs: Optional[ObservabilityLogs] = None
    """Log settings for the Worker."""

    traces: Optional[ObservabilityTraces] = None
    """Trace settings for the Worker."""


class ScriptSetting(BaseModel):
    logpush: Optional[bool] = None
    """Whether Logpush is turned on for the Worker."""

    observability: Optional[Observability] = None
    """Observability settings for the Worker."""

    tags: Optional[List[str]] = None
    """Tags associated with the Worker."""

    tail_consumers: Optional[List[ConsumerScript]] = None
    """List of Workers that will consume logs from the attached Worker."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...types.ips import ip_list_params
from ..._base_client import make_request_options
from ...types.ips.ip_list_response import IPListResponse

__all__ = ["IPsResource", "AsyncIPsResource"]


class IPsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> IPsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return IPsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IPsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return IPsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        networks: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IPListResponse]:
        """
        Get IPs used on the Cloudflare/JD Cloud network, see
        https://www.cloudflare.com/ips for Cloudflare IPs or
        https://developers.cloudflare.com/china-network/reference/infrastructure/ for JD
        Cloud IPs.

        Args:
          networks: Specified as `jdcloud` to list IPs used by JD Cloud data centers.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return cast(
            Optional[IPListResponse],
            self._get(
                "/ips",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    query=maybe_transform({"networks": networks}, ip_list_params.IPListParams),
                    post_parser=ResultWrapper[Optional[IPListResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[IPListResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncIPsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncIPsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncIPsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIPsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncIPsResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        networks: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IPListResponse]:
        """
        Get IPs used on the Cloudflare/JD Cloud network, see
        https://www.cloudflare.com/ips for Cloudflare IPs or
        https://developers.cloudflare.com/china-network/reference/infrastructure/ for JD
        Cloud IPs.

        Args:
          networks: Specified as `jdcloud` to list IPs used by JD Cloud data centers.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return cast(
            Optional[IPListResponse],
            await self._get(
                "/ips",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    query=await async_maybe_transform({"networks": networks}, ip_list_params.IPListParams),
                    post_parser=ResultWrapper[Optional[IPListResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[IPListResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class IPsResourceWithRawResponse:
    def __init__(self, ips: IPsResource) -> None:
        self._ips = ips

        self.list = to_raw_response_wrapper(
            ips.list,
        )


class AsyncIPsResourceWithRawResponse:
    def __init__(self, ips: AsyncIPsResource) -> None:
        self._ips = ips

        self.list = async_to_raw_response_wrapper(
            ips.list,
        )


class IPsResourceWithStreamingResponse:
    def __init__(self, ips: IPsResource) -> None:
        self._ips = ips

        self.list = to_streamed_response_wrapper(
            ips.list,
        )


class AsyncIPsResourceWithStreamingResponse:
    def __init__(self, ips: AsyncIPsResource) -> None:
        self._ips = ips

        self.list = async_to_streamed_response_wrapper(
            ips.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["AppListParams"]


class AppListParams(TypedDict, total=False):
    zone_id: Required[str]
    """Zone identifier."""

    direction: Literal["asc", "desc"]
    """Sets the direction by which results are ordered."""

    order: Literal["protocol", "app_id", "created_on", "modified_on", "dns"]
    """Application field by which results are ordered."""

    page: float
    """Page number of paginated results.

    This parameter is required in order to use other pagination parameters. If
    included in the query, `result_info` will be present in the response.
    """

    per_page: float
    """Sets the maximum number of results per page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["CredentialEditParams"]


class CredentialEditParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    credential_set_id: Required[str]

    location: Literal["header", "cookie"]
    """Where the credential is attached in outgoing requests."""

    location_name: str
    """Name of the header or cookie where the credential is attached."""

    name: str
    """Human-readable name."""

    value: str
    """The credential value. Write-only. Never returned in responses."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["TokenPolicy", "PermissionGroup", "PermissionGroupMeta"]


class PermissionGroupMeta(BaseModel):
    """Attributes associated to the permission group."""

    key: Optional[str] = None

    value: Optional[str] = None


class PermissionGroup(BaseModel):
    """
    A named group of permissions that map to a group of operations against resources.
    """

    id: str
    """Identifier of the permission group."""

    meta: Optional[PermissionGroupMeta] = None
    """Attributes associated to the permission group."""

    name: Optional[str] = None
    """Name of the permission group."""


class TokenPolicy(BaseModel):
    id: str
    """Policy identifier."""

    effect: Literal["allow", "deny"]
    """Allow or deny operations against the resources."""

    permission_groups: List[PermissionGroup]
    """A set of permission groups that are specified to the policy."""

    resources: Union[Dict[str, str], Dict[str, Dict[str, str]]]
    """A list of resource names that the policy applies to."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = ["KeyBulkGetParams"]


class KeyBulkGetParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    keys: Required[SequenceNotStr[str]]
    """Array of keys to retrieve (maximum of 100)."""

    type: Literal["text", "json"]
    """Whether to parse JSON values in the response."""

    with_metadata: Annotated[bool, PropertyInfo(alias="withMetadata")]
    """Whether to include metadata in the response."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel

__all__ = ["DynamicRoutingListDeploymentsResponse", "Data", "DataDeployment"]


class DataDeployment(BaseModel):
    created_at: str

    deployment_id: str

    version_id: str


class Data(BaseModel):
    deployments: List[DataDeployment]

    order_by: str

    order_by_direction: str

    page: float

    per_page: float


class DynamicRoutingListDeploymentsResponse(BaseModel):
    data: Data

    success: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Union, Optional, cast
from datetime import datetime

import httpx

from .bytimes import (
    BytimesResource,
    AsyncBytimesResource,
    BytimesResourceWithRawResponse,
    AsyncBytimesResourceWithRawResponse,
    BytimesResourceWithStreamingResponse,
    AsyncBytimesResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.dns.analytics import report_get_params
from .....types.dns.analytics.report import Report

__all__ = ["ReportsResource", "AsyncReportsResource"]


class ReportsResource(SyncAPIResource):
    @cached_property
    def bytimes(self) -> BytimesResource:
        return BytimesResource(self._client)

    @cached_property
    def with_raw_response(self) -> ReportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ReportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ReportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ReportsResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        zone_id: str,
        dimensions: str | Omit = omit,
        filters: str | Omit = omit,
        limit: int | Omit = omit,
        metrics: str | Omit = omit,
        since: Union[str, datetime] | Omit = omit,
        sort: str | Omit = omit,
        until: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Report]:
        """
        Retrieves a list of summarised aggregate metrics over a given time period.

        See
        [Analytics API properties](https://developers.cloudflare.com/dns/reference/analytics-api-properties/)
        for detailed information about the available query parameters.

        Args:
          zone_id: Identifier.

          dimensions: A comma-separated list of dimensions to group results by.

          filters: Segmentation filter in 'attribute operator value' format.

          limit: Limit number of returned metrics.

          metrics: A comma-separated list of metrics to query.

          since: Start date and time of requesting data period in ISO 8601 format.

          sort: A comma-separated list of dimensions to sort by, where each dimension may be
              prefixed by - (descending) or + (ascending).

          until: End date and time of requesting data period in ISO 8601 format.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/dns_analytics/report", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "dimensions": dimensions,
                        "filters": filters,
                        "limit": limit,
                        "metrics": metrics,
                        "since": since,
                        "sort": sort,
                        "until": until,
                    },
                    report_get_params.ReportGetParams,
                ),
                post_parser=ResultWrapper[Optional[Report]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Report]], ResultWrapper[Report]),
        )


class AsyncReportsResource(AsyncAPIResource):
    @cached_property
    def bytimes(self) -> AsyncBytimesResource:
        return AsyncBytimesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncReportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncReportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncReportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncReportsResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        zone_id: str,
        dimensions: str | Omit = omit,
        filters: str | Omit = omit,
        limit: int | Omit = omit,
        metrics: str | Omit = omit,
        since: Union[str, datetime] | Omit = omit,
        sort: str | Omit = omit,
        until: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Report]:
        """
        Retrieves a list of summarised aggregate metrics over a given time period.

        See
        [Analytics API properties](https://developers.cloudflare.com/dns/reference/analytics-api-properties/)
        for detailed information about the available query parameters.

        Args:
          zone_id: Identifier.

          dimensions: A comma-separated list of dimensions to group results by.

          filters: Segmentation filter in 'attribute operator value' format.

          limit: Limit number of returned metrics.

          metrics: A comma-separated list of metrics to query.

          since: Start date and time of requesting data period in ISO 8601 format.

          sort: A comma-separated list of dimensions to sort by, where each dimension may be
              prefixed by - (descending) or + (ascending).

          until: End date and time of requesting data period in ISO 8601 format.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/dns_analytics/report", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "dimensions": dimensions,
                        "filters": filters,
                        "limit": limit,
                        "metrics": metrics,
                        "since": since,
                        "sort": sort,
                        "until": until,
                    },
                    report_get_params.ReportGetParams,
                ),
                post_parser=ResultWrapper[Optional[Report]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Report]], ResultWrapper[Report]),
        )


class ReportsResourceWithRawResponse:
    def __init__(self, reports: ReportsResource) -> None:
        self._reports = reports

        self.get = to_raw_response_wrapper(
            reports.get,
        )

    @cached_property
    def bytimes(self) -> BytimesResourceWithRawResponse:
        return BytimesResourceWithRawResponse(self._reports.bytimes)


class AsyncReportsResourceWithRawResponse:
    def __init__(self, reports: AsyncReportsResource) -> None:
        self._reports = reports

        self.get = async_to_raw_response_wrapper(
            reports.get,
        )

    @cached_property
    def bytimes(self) -> AsyncBytimesResourceWithRawResponse:
        return AsyncBytimesResourceWithRawResponse(self._reports.bytimes)


class ReportsResourceWithStreamingResponse:
    def __init__(self, reports: ReportsResource) -> None:
        self._reports = reports

        self.get = to_streamed_response_wrapper(
            reports.get,
        )

    @cached_property
    def bytimes(self) -> BytimesResourceWithStreamingResponse:
        return BytimesResourceWithStreamingResponse(self._reports.bytimes)


class AsyncReportsResourceWithStreamingResponse:
    def __init__(self, reports: AsyncReportsResource) -> None:
        self._reports = reports

        self.get = async_to_streamed_response_wrapper(
            reports.get,
        )

    @cached_property
    def bytimes(self) -> AsyncBytimesResourceWithStreamingResponse:
        return AsyncBytimesResourceWithStreamingResponse(self._reports.bytimes)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["Message"]


class Message(BaseModel):
    id: int
    """Message ID."""

    author: str
    """Author of message."""

    content: str
    """Content of message."""

    is_follow_on_request: bool
    """Whether the message is a follow-on request."""

    updated: datetime
    """Defines the message last updated time."""

    created: Optional[datetime] = None
    """Defines the message creation time."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal, TypedDict

from ..._types import SequenceNotStr

__all__ = ["HTTPConfigurationParam"]


class HTTPConfigurationParam(TypedDict, total=False):
    """Parameters specific to an HTTP or HTTPS health check."""

    allow_insecure: bool
    """Do not validate the certificate when the health check uses HTTPS."""

    expected_body: str
    """A case-insensitive sub-string to look for in the response body.

    If this string is not found, the origin will be marked as unhealthy.
    """

    expected_codes: Optional[SequenceNotStr[str]]
    """The expected HTTP response codes (e.g.

    "200") or code ranges (e.g. "2xx" for all codes starting with 2) of the health
    check.
    """

    follow_redirects: bool
    """Follow redirects if the origin returns a 3xx status code."""

    header: Optional[Dict[str, SequenceNotStr[str]]]
    """The HTTP request headers to send in the health check.

    It is recommended you set a Host header by default. The User-Agent header cannot
    be overridden.
    """

    method: Literal["GET", "HEAD"]
    """The HTTP method to use for the health check."""

    path: str
    """The endpoint path to health check against."""

    port: int
    """Port number to connect to for the health check.

    Defaults to 80 if type is HTTP or 443 if type is HTTPS.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["MeetingEditParticipantParams"]


class MeetingEditParticipantParams(TypedDict, total=False):
    account_id: Required[str]
    """The account identifier tag."""

    app_id: Required[str]
    """The app identifier tag."""

    meeting_id: Required[str]

    name: Optional[str]
    """(Optional) Name of the participant."""

    picture: Optional[str]
    """(Optional) A URL to a picture to be used for the participant."""

    preset_name: Optional[str]
    """(Optional) Name of the preset to apply to this participant."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["BillingHistory", "Zone"]


class Zone(BaseModel):
    name: Optional[str] = None


class BillingHistory(BaseModel):
    id: str
    """Billing item identifier tag."""

    action: str
    """The billing item action."""

    amount: float
    """The amount associated with this billing item."""

    currency: str
    """The monetary unit in which pricing information is displayed."""

    description: str
    """The billing item description."""

    occurred_at: datetime
    """When the billing item was created."""

    type: str
    """The billing item type."""

    zone: Zone
